# coding: utf-8

"""
    Core Operations

    The Sparkfly Platform provides a full lifecycle for promotions and rewards from creation to distribution to settlement. The platform integrates in real-time at the point-of-sale and provides item level discounting and tracking. The capabilities of the Sparkfly Platform are available through the use of the Sparkfly Platform API.  The Sparkfly documentation site is under development. If the documentation you're after isn't available here, please contact support@sparkfly.com and we will get you what you need.

    The version of the OpenAPI document: 1.0
    Contact: support@sparkfly.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr, field_validator
from typing import Any, List, Optional
from typing_extensions import Annotated
from sparkfly.models.offer import Offer
from sparkfly.models.offer_index_body import OfferIndexBody
from sparkfly.models.offer_request import OfferRequest

from sparkfly.api_client import ApiClient, RequestSerialized
from sparkfly.api_response import ApiResponse
from sparkfly.rest import RESTResponseType


class OffersApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    async def create_offer(
        self,
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Offer:
        """Create an Offer

        Creates an offer. redemption_grace_period is the grace period provided for generating and redeeming codes.  If not provided. The default will be the related Campaign's End Date. If not provided however a grace period exists on the Campaign. The grace period will be the Campaign End Date + Campaign Grace Period Days.  In all cases, the Campaign End Date or Offer End Date - which ever occurs first - plus any applicable grace period days.  (ex: You cannot set a campaign end date beyond the offer expiration date. In this case it will follow the offer expiration date)

        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_offer_serialize(
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "Offer",
            '400': "Offer",
            '401': None,
            '422': "Offer",
            '500': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def create_offer_with_http_info(
        self,
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Offer]:
        """Create an Offer

        Creates an offer. redemption_grace_period is the grace period provided for generating and redeeming codes.  If not provided. The default will be the related Campaign's End Date. If not provided however a grace period exists on the Campaign. The grace period will be the Campaign End Date + Campaign Grace Period Days.  In all cases, the Campaign End Date or Offer End Date - which ever occurs first - plus any applicable grace period days.  (ex: You cannot set a campaign end date beyond the offer expiration date. In this case it will follow the offer expiration date)

        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_offer_serialize(
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "Offer",
            '400': "Offer",
            '401': None,
            '422': "Offer",
            '500': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def create_offer_without_preload_content(
        self,
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create an Offer

        Creates an offer. redemption_grace_period is the grace period provided for generating and redeeming codes.  If not provided. The default will be the related Campaign's End Date. If not provided however a grace period exists on the Campaign. The grace period will be the Campaign End Date + Campaign Grace Period Days.  In all cases, the Campaign End Date or Offer End Date - which ever occurs first - plus any applicable grace period days.  (ex: You cannot set a campaign end date beyond the offer expiration date. In this case it will follow the offer expiration date)

        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_offer_serialize(
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "Offer",
            '400': "Offer",
            '401': None,
            '422': "Offer",
            '500': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_offer_serialize(
        self,
        content_type,
        offer_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        if content_type is not None:
            _header_params['Content-Type'] = content_type
        # process the form parameters
        # process the body parameter
        if offer_request is not None:
            _body_params = offer_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'XAuthToken'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/offers',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def delete_offer(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete an Offer

        Deletes an offer by ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def delete_offer_with_http_info(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete an Offer

        Deletes an offer by ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def delete_offer_without_preload_content(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete an Offer

        Deletes an offer by ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_offer_serialize(
        self,
        offer_id,
        content_type,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if offer_id is not None:
            _path_params['offer_id'] = offer_id
        # process the query parameters
        # process the header parameters
        if content_type is not None:
            _header_params['Content-Type'] = content_type
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'XAuthToken'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/offers/{offer_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def get_offer(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Offer:
        """Find an Offer

        Finds an offer by the ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Offer",
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def get_offer_with_http_info(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Offer]:
        """Find an Offer

        Finds an offer by the ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Offer",
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def get_offer_without_preload_content(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Find an Offer

        Finds an offer by the ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Offer",
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_offer_serialize(
        self,
        offer_id,
        content_type,
        offer_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if offer_id is not None:
            _path_params['offer_id'] = offer_id
        # process the query parameters
        # process the header parameters
        if content_type is not None:
            _header_params['Content-Type'] = content_type
        # process the form parameters
        # process the body parameter
        if offer_request is not None:
            _body_params = offer_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'XAuthToken'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/offers/{offer_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def list_offers(
        self,
        page: Annotated[Optional[Annotated[str, Field(min_length=1, strict=True)]], Field(description="page offset to display a range of records from")] = None,
        per_page: Annotated[Optional[Annotated[str, Field(min_length=1, strict=True)]], Field(description="maximum number of records to return in the search")] = None,
        order: Annotated[Optional[StrictStr], Field(description="sort the records in either ascending (asc) or descending (desc) order")] = None,
        search_text: Annotated[Optional[StrictStr], Field(description="Attempts to match by name or id")] = None,
        sort_by: Annotated[Optional[StrictStr], Field(description="sort records by a field name")] = None,
        status: Annotated[Optional[StrictStr], Field(description="search for records matching the desired status")] = None,
        merchant_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching merchant ID")] = None,
        manufacturer_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching manufacturer ID")] = None,
        store_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching store ID")] = None,
        offer_list_id: Annotated[Optional[List[Any]], Field(description="search for records with a matching offer list ID")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> OfferIndexBody:
        """List all Offers

        Search for offers.

        :param page: page offset to display a range of records from
        :type page: str
        :param per_page: maximum number of records to return in the search
        :type per_page: str
        :param order: sort the records in either ascending (asc) or descending (desc) order
        :type order: str
        :param search_text: Attempts to match by name or id
        :type search_text: str
        :param sort_by: sort records by a field name
        :type sort_by: str
        :param status: search for records matching the desired status
        :type status: str
        :param merchant_id: search for records with a matching merchant ID
        :type merchant_id: int
        :param manufacturer_id: search for records with a matching manufacturer ID
        :type manufacturer_id: int
        :param store_id: search for records with a matching store ID
        :type store_id: int
        :param offer_list_id: search for records with a matching offer list ID
        :type offer_list_id: List[object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_offers_serialize(
            page=page,
            per_page=per_page,
            order=order,
            search_text=search_text,
            sort_by=sort_by,
            status=status,
            merchant_id=merchant_id,
            manufacturer_id=manufacturer_id,
            store_id=store_id,
            offer_list_id=offer_list_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "OfferIndexBody",
            '401': None,
            '500': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def list_offers_with_http_info(
        self,
        page: Annotated[Optional[Annotated[str, Field(min_length=1, strict=True)]], Field(description="page offset to display a range of records from")] = None,
        per_page: Annotated[Optional[Annotated[str, Field(min_length=1, strict=True)]], Field(description="maximum number of records to return in the search")] = None,
        order: Annotated[Optional[StrictStr], Field(description="sort the records in either ascending (asc) or descending (desc) order")] = None,
        search_text: Annotated[Optional[StrictStr], Field(description="Attempts to match by name or id")] = None,
        sort_by: Annotated[Optional[StrictStr], Field(description="sort records by a field name")] = None,
        status: Annotated[Optional[StrictStr], Field(description="search for records matching the desired status")] = None,
        merchant_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching merchant ID")] = None,
        manufacturer_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching manufacturer ID")] = None,
        store_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching store ID")] = None,
        offer_list_id: Annotated[Optional[List[Any]], Field(description="search for records with a matching offer list ID")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[OfferIndexBody]:
        """List all Offers

        Search for offers.

        :param page: page offset to display a range of records from
        :type page: str
        :param per_page: maximum number of records to return in the search
        :type per_page: str
        :param order: sort the records in either ascending (asc) or descending (desc) order
        :type order: str
        :param search_text: Attempts to match by name or id
        :type search_text: str
        :param sort_by: sort records by a field name
        :type sort_by: str
        :param status: search for records matching the desired status
        :type status: str
        :param merchant_id: search for records with a matching merchant ID
        :type merchant_id: int
        :param manufacturer_id: search for records with a matching manufacturer ID
        :type manufacturer_id: int
        :param store_id: search for records with a matching store ID
        :type store_id: int
        :param offer_list_id: search for records with a matching offer list ID
        :type offer_list_id: List[object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_offers_serialize(
            page=page,
            per_page=per_page,
            order=order,
            search_text=search_text,
            sort_by=sort_by,
            status=status,
            merchant_id=merchant_id,
            manufacturer_id=manufacturer_id,
            store_id=store_id,
            offer_list_id=offer_list_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "OfferIndexBody",
            '401': None,
            '500': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def list_offers_without_preload_content(
        self,
        page: Annotated[Optional[Annotated[str, Field(min_length=1, strict=True)]], Field(description="page offset to display a range of records from")] = None,
        per_page: Annotated[Optional[Annotated[str, Field(min_length=1, strict=True)]], Field(description="maximum number of records to return in the search")] = None,
        order: Annotated[Optional[StrictStr], Field(description="sort the records in either ascending (asc) or descending (desc) order")] = None,
        search_text: Annotated[Optional[StrictStr], Field(description="Attempts to match by name or id")] = None,
        sort_by: Annotated[Optional[StrictStr], Field(description="sort records by a field name")] = None,
        status: Annotated[Optional[StrictStr], Field(description="search for records matching the desired status")] = None,
        merchant_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching merchant ID")] = None,
        manufacturer_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching manufacturer ID")] = None,
        store_id: Annotated[Optional[StrictInt], Field(description="search for records with a matching store ID")] = None,
        offer_list_id: Annotated[Optional[List[Any]], Field(description="search for records with a matching offer list ID")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List all Offers

        Search for offers.

        :param page: page offset to display a range of records from
        :type page: str
        :param per_page: maximum number of records to return in the search
        :type per_page: str
        :param order: sort the records in either ascending (asc) or descending (desc) order
        :type order: str
        :param search_text: Attempts to match by name or id
        :type search_text: str
        :param sort_by: sort records by a field name
        :type sort_by: str
        :param status: search for records matching the desired status
        :type status: str
        :param merchant_id: search for records with a matching merchant ID
        :type merchant_id: int
        :param manufacturer_id: search for records with a matching manufacturer ID
        :type manufacturer_id: int
        :param store_id: search for records with a matching store ID
        :type store_id: int
        :param offer_list_id: search for records with a matching offer list ID
        :type offer_list_id: List[object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_offers_serialize(
            page=page,
            per_page=per_page,
            order=order,
            search_text=search_text,
            sort_by=sort_by,
            status=status,
            merchant_id=merchant_id,
            manufacturer_id=manufacturer_id,
            store_id=store_id,
            offer_list_id=offer_list_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "OfferIndexBody",
            '401': None,
            '500': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_offers_serialize(
        self,
        page,
        per_page,
        order,
        search_text,
        sort_by,
        status,
        merchant_id,
        manufacturer_id,
        store_id,
        offer_list_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'offer_list_id': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if per_page is not None:
            
            _query_params.append(('per_page', per_page))
            
        if order is not None:
            
            _query_params.append(('order', order))
            
        if search_text is not None:
            
            _query_params.append(('search_text', search_text))
            
        if sort_by is not None:
            
            _query_params.append(('sort_by', sort_by))
            
        if status is not None:
            
            _query_params.append(('status', status))
            
        if merchant_id is not None:
            
            _query_params.append(('merchant_id', merchant_id))
            
        if manufacturer_id is not None:
            
            _query_params.append(('manufacturer_id', manufacturer_id))
            
        if store_id is not None:
            
            _query_params.append(('store_id', store_id))
            
        if offer_list_id is not None:
            
            _query_params.append(('offer_list_id', offer_list_id))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'XAuthToken'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/offers',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def update_offer(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Offer:
        """Update an Offer

        Updates an offer by ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Offer",
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def update_offer_with_http_info(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Offer]:
        """Update an Offer

        Updates an offer by ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Offer",
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def update_offer_without_preload_content(
        self,
        offer_id: Annotated[StrictStr, Field(description="Primary ID of the offer")],
        content_type: Annotated[Optional[StrictStr], Field(description="application/json")] = None,
        offer_request: Optional[OfferRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update an Offer

        Updates an offer by ID.

        :param offer_id: Primary ID of the offer (required)
        :type offer_id: str
        :param content_type: application/json
        :type content_type: str
        :param offer_request:
        :type offer_request: OfferRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_offer_serialize(
            offer_id=offer_id,
            content_type=content_type,
            offer_request=offer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Offer",
            '401': None,
            '404': "ApiError",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_offer_serialize(
        self,
        offer_id,
        content_type,
        offer_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if offer_id is not None:
            _path_params['offer_id'] = offer_id
        # process the query parameters
        # process the header parameters
        if content_type is not None:
            _header_params['Content-Type'] = content_type
        # process the form parameters
        # process the body parameter
        if offer_request is not None:
            _body_params = offer_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'XAuthToken'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/offers/{offer_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Core Operations

    The Sparkfly Platform provides a full lifecycle for promotions and rewards from creation to distribution to settlement. The platform integrates in real-time at the point-of-sale and provides item level discounting and tracking. The capabilities of the Sparkfly Platform are available through the use of the Sparkfly Platform API.  The Sparkfly documentation site is under development. If the documentation you're after isn't available here, please contact support@sparkfly.com and we will get you what you need.

    The version of the OpenAPI document: 1.0
    Contact: support@sparkfly.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from sparkfly.api.eligible_item_sets_api import EligibleItemSetsApi


class TestEligibleItemSetsApi(unittest.IsolatedAsyncioTestCase):
    """EligibleItemSetsApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = EligibleItemSetsApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_eligible_item_set(self) -> None:
        """Test case for create_eligible_item_set

        Create an Eligible Item Set
        """
        pass

    async def test_delete_eligible_item_set(self) -> None:
        """Test case for delete_eligible_item_set

        Remove an Eligible Item Set
        """
        pass

    async def test_get_eligible_item_set(self) -> None:
        """Test case for get_eligible_item_set

        Get an Eligible Item Set
        """
        pass

    async def test_list_eligible_item_sets(self) -> None:
        """Test case for list_eligible_item_sets

        List all Eligible Item Sets
        """
        pass

    async def test_update_eligible_item_set(self) -> None:
        """Test case for update_eligible_item_set

        Update an Eligible Item Set
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Core Operations

    The Sparkfly Platform provides a full lifecycle for promotions and rewards from creation to distribution to settlement. The platform integrates in real-time at the point-of-sale and provides item level discounting and tracking. The capabilities of the Sparkfly Platform are available through the use of the Sparkfly Platform API.  The Sparkfly documentation site is under development. If the documentation you're after isn't available here, please contact support@sparkfly.com and we will get you what you need.

    The version of the OpenAPI document: 1.0
    Contact: support@sparkfly.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from sparkfly.api.ctm_api import CtmApi


class TestCtmApi(unittest.IsolatedAsyncioTestCase):
    """CtmApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = CtmApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_ctm_allocate(self) -> None:
        """Test case for ctm_allocate

        CTM Allocate
        """
        pass

    async def test_ctm_create_member(self) -> None:
        """Test case for ctm_create_member

        CTM Update Response
        """
        pass

    async def test_ctm_create_member_by_site(self) -> None:
        """Test case for ctm_create_member_by_site

        CTM Update Response
        """
        pass

    async def test_ctm_deallocate(self) -> None:
        """Test case for ctm_deallocate

        CTM Dealocate
        """
        pass

    async def test_ctm_get_custom_messages(self) -> None:
        """Test case for ctm_get_custom_messages

        CTM Custom Messages
        """
        pass

    async def test_ctm_get_member(self) -> None:
        """Test case for ctm_get_member

        CTM Show Response
        """
        pass

    async def test_ctm_get_qitems(self) -> None:
        """Test case for ctm_get_qitems

        CTM Qitems
        """
        pass

    async def test_ctm_get_store(self) -> None:
        """Test case for ctm_get_store

        CTM Stores
        """
        pass

    async def test_ctm_update_member(self) -> None:
        """Test case for ctm_update_member

        CTM Update Response
        """
        pass

    async def test_ctm_update_member_v2(self) -> None:
        """Test case for ctm_update_member_v2

        CTM Show Response
        """
        pass

    async def test_ctm_update_transaction(self) -> None:
        """Test case for ctm_update_transaction

        CTM Transaction Update
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Core Operations

    The Sparkfly Platform provides a full lifecycle for promotions and rewards from creation to distribution to settlement. The platform integrates in real-time at the point-of-sale and provides item level discounting and tracking. The capabilities of the Sparkfly Platform are available through the use of the Sparkfly Platform API.  The Sparkfly documentation site is under development. If the documentation you're after isn't available here, please contact support@sparkfly.com and we will get you what you need.

    The version of the OpenAPI document: 1.0
    Contact: support@sparkfly.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from sparkfly.api.items_api import ItemsApi


class TestItemsApi(unittest.IsolatedAsyncioTestCase):
    """ItemsApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = ItemsApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_add_item_to_set(self) -> None:
        """Test case for add_item_to_set

        Add Item to Set
        """
        pass

    async def test_create_item(self) -> None:
        """Test case for create_item

        Create Item
        """
        pass

    async def test_create_item_set(self) -> None:
        """Test case for create_item_set

        Item Set Create
        """
        pass

    async def test_delete_item(self) -> None:
        """Test case for delete_item

        Delete Item
        """
        pass

    async def test_delete_item_set(self) -> None:
        """Test case for delete_item_set

        Delete an Item Set
        """
        pass

    async def test_get_item(self) -> None:
        """Test case for get_item

        Get Item
        """
        pass

    async def test_get_item_set(self) -> None:
        """Test case for get_item_set

        Get an Item Set
        """
        pass

    async def test_list_item_sets(self) -> None:
        """Test case for list_item_sets

        Item Set Index
        """
        pass

    async def test_list_items(self) -> None:
        """Test case for list_items

        Item Index
        """
        pass

    async def test_list_set_items(self) -> None:
        """Test case for list_set_items

        Item Set Item Index
        """
        pass

    async def test_remove_item_from_set(self) -> None:
        """Test case for remove_item_from_set

        Remove Item from Set
        """
        pass

    async def test_update_item(self) -> None:
        """Test case for update_item

        Update Item
        """
        pass

    async def test_update_item_set(self) -> None:
        """Test case for update_item_set

        Update an Item Set
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Core Operations

    The Sparkfly Platform provides a full lifecycle for promotions and rewards from creation to distribution to settlement. The platform integrates in real-time at the point-of-sale and provides item level discounting and tracking. The capabilities of the Sparkfly Platform are available through the use of the Sparkfly Platform API.  The Sparkfly documentation site is under development. If the documentation you're after isn't available here, please contact support@sparkfly.com and we will get you what you need.

    The version of the OpenAPI document: 1.0
    Contact: support@sparkfly.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from sparkfly.api.offer_pos_offer_codes_api import OfferPOSOfferCodesApi


class TestOfferPOSOfferCodesApi(unittest.IsolatedAsyncioTestCase):
    """OfferPOSOfferCodesApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = OfferPOSOfferCodesApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_pos_code(self) -> None:
        """Test case for create_pos_code

        Create an Offer POS Offer Code
        """
        pass

    async def test_delete_pos_code(self) -> None:
        """Test case for delete_pos_code

        Remove an Offer POS Offer Code
        """
        pass

    async def test_get_offer_pos_code(self) -> None:
        """Test case for get_offer_pos_code

        Get an Offer POS Offer Code
        """
        pass

    async def test_list_pos_codes(self) -> None:
        """Test case for list_pos_codes

        List all Offer POS Offer Codes
        """
        pass

    async def test_update_pos_code(self) -> None:
        """Test case for update_pos_code

        Update an Offer POS Offer Code
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Core Operations

    The Sparkfly Platform provides a full lifecycle for promotions and rewards from creation to distribution to settlement. The platform integrates in real-time at the point-of-sale and provides item level discounting and tracking. The capabilities of the Sparkfly Platform are available through the use of the Sparkfly Platform API.  The Sparkfly documentation site is under development. If the documentation you're after isn't available here, please contact support@sparkfly.com and we will get you what you need.

    The version of the OpenAPI document: 1.0
    Contact: support@sparkfly.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from sparkfly.api.offers_api import OffersApi


class TestOffersApi(unittest.IsolatedAsyncioTestCase):
    """OffersApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = OffersApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_create_offer(self) -> None:
        """Test case for create_offer

        Create an Offer
        """
        pass

    async def test_delete_offer(self) -> None:
        """Test case for delete_offer

        Delete an Offer
        """
        pass

    async def test_get_offer(self) -> None:
        """Test case for get_offer

        Find an Offer
        """
        pass

    async def test_list_offers(self) -> None:
        """Test case for list_offers

        List all Offers
        """
        pass

    async def test_update_offer(self) -> None:
        """Test case for update_offer

        Update an Offer
        """
        pass


if __name__ == '__main__':
    unittest.main()



"""
Sparkfly API Client

A simplified wrapper around the generated OpenAPI client.
"""

import asyncio
import time
from typing import Optional, Callable
from .api_client import ApiClient
from .configuration import Configuration
from .api import (
    AuthenticationApi,
    CampaignsApi,
    StoresApi,
    OffersApi,
    OfferStatesApi,
    MembersApi,
    ItemsApi,
    OfferListsApi,
    ImpressionsApi,
    EmailOptInApi,
    TemplatesApi,
    AudiencesApi,
    BIStoreListsApi,
    CtmApi,
    EligibleItemSetsApi,
    MemberPrivacyApi,
    OfferPOSOfferCodesApi,
    POSOfferCodesApi,
    StoreListsApi,
    EventNotificationsApi,
    TransactionsApi,
)


class RetryConfig:
    """Configuration for retry behavior."""

    def __init__(
        self,
        max_retries: int = 3,
        base_delay: float = 1.0,
        max_delay: float = 60.0,
        backoff_factor: float = 2.0,
        retry_on_exceptions: tuple = (Exception,),
        retry_condition: Optional[Callable[[Exception], bool]] = None,
    ):
        """
        Initialize retry configuration.

        Args:
            max_retries: Maximum number of retry attempts
            base_delay: Initial delay between retries in seconds
            max_delay: Maximum delay between retries in seconds
            backoff_factor: Multiplier for delay after each retry
            retry_on_exceptions: Tuple of exceptions to retry on
            retry_condition: Optional function to determine if retry should occur
        """
        self.max_retries = max_retries
        self.base_delay = base_delay
        self.max_delay = max_delay
        self.backoff_factor = backoff_factor
        self.retry_on_exceptions = retry_on_exceptions
        self.retry_condition = retry_condition


class Sparkfly:
    """
    A simplified async client for the Sparkfly Platform API.

    This client handles authentication automatically and provides
    easy access to all API endpoints with built-in retry logic.
    """

    def __init__(
        self,
        identity: str,
        key: str,
        environment: str = "staging",
        host: Optional[str] = None,
        token: Optional[str] = None,
        token_expires_at: Optional[float] = None,
        retry_config: Optional[RetryConfig] = None,
    ):
        """
        Initialize the Sparkfly client.

        Args:
            identity: Your Sparkfly account identity
            key: Your Sparkfly account secret key
            environment: The environment to use ('staging' or 'production')
            host: Optional custom host URL (overrides environment).
                 Must be one of the valid Sparkfly base URLs:
                 - https://api-staging.sparkfly.com (staging)
                 - https://api.sparkfly.com (production)
                 The /v1.0 suffix will be added automatically
            token: Optional pre-existing auth token
            token_expires_at: Optional token expiration timestamp
            retry_config: Optional retry configuration (defaults to 3 retries, 1s delay)
        """
        self.identity = identity
        self.key = key
        self.environment = environment
        self.retry_config = retry_config or RetryConfig()

        # Determine the host URL
        if host:
            # Validate that the host is one of the valid base URLs
            valid_base_hosts = [
                "https://api-staging.sparkfly.com",
                "https://api.sparkfly.com",
            ]
            if host not in valid_base_hosts:
                raise ValueError(
                    f"Invalid host URL. Must be one of: {', '.join(valid_base_hosts)}"
                )

            # Always append /v1.0 to the base URL
            self.host = f"{host}/v1.0"
        elif environment.lower() == "production":
            self.host = "https://api.sparkfly.com/v1.0"
        elif environment.lower() == "staging":
            self.host = "https://api-staging.sparkfly.com/v1.0"
        else:
            raise ValueError("Environment must be 'staging' or 'production'")

        self._token = token
        self._token_expires_at = token_expires_at

        # Initialize the API client
        self._config = Configuration(
            host=self.host,
            server_index=1 if self.environment.lower() == "production" else 0,
            api_key={
                "XAuthIdentity": identity,
                "XAuthKey": key,
            },
        )
        self._api_client = ApiClient(configuration=self._config)

        # Initialize API classes with retry wrappers
        self.auth: AuthenticationApi = self._create_retry_wrapper(AuthenticationApi)
        self.campaigns: CampaignsApi = self._create_retry_wrapper(CampaignsApi)
        self.stores: StoresApi = self._create_retry_wrapper(StoresApi)
        self.offers: OffersApi = self._create_retry_wrapper(OffersApi)
        self.offer_states: OfferStatesApi = self._create_retry_wrapper(OfferStatesApi)
        self.members: MembersApi = self._create_retry_wrapper(MembersApi)
        self.items: ItemsApi = self._create_retry_wrapper(ItemsApi)
        self.offer_lists: OfferListsApi = self._create_retry_wrapper(OfferListsApi)
        self.impressions: ImpressionsApi = self._create_retry_wrapper(ImpressionsApi)
        self.email_opt_in: EmailOptInApi = self._create_retry_wrapper(EmailOptInApi)
        self.templates: TemplatesApi = self._create_retry_wrapper(TemplatesApi)
        self.audiences: AudiencesApi = self._create_retry_wrapper(AudiencesApi)
        self.bi_store_lists: BIStoreListsApi = self._create_retry_wrapper(
            BIStoreListsApi
        )
        self.ctm: CtmApi = self._create_retry_wrapper(CtmApi)
        self.eligible_item_sets: EligibleItemSetsApi = self._create_retry_wrapper(
            EligibleItemSetsApi
        )
        self.member_privacy: MemberPrivacyApi = self._create_retry_wrapper(
            MemberPrivacyApi
        )
        self.offer_pos_offer_codes: OfferPOSOfferCodesApi = self._create_retry_wrapper(
            OfferPOSOfferCodesApi
        )
        self.pos_offer_codes: POSOfferCodesApi = self._create_retry_wrapper(
            POSOfferCodesApi
        )
        self.store_lists: StoreListsApi = self._create_retry_wrapper(StoreListsApi)
        self.event_notifications: EventNotificationsApi = self._create_retry_wrapper(
            EventNotificationsApi
        )
        self.transactions: TransactionsApi = self._create_retry_wrapper(TransactionsApi)

    def _create_retry_wrapper(self, api_class):
        """Create a wrapper class that automatically applies retry logic to all methods."""

        class RetryWrappedApi(api_class):
            def __init__(self, api_client, sparkfly_client):
                super().__init__(api_client)
                self._sparkfly_client = sparkfly_client

            def __getattribute__(self, name):
                """Override to wrap async methods with retry logic."""
                # Use object.__getattribute__ to avoid recursion
                attr = object.__getattribute__(self, name)

                # Only wrap async methods that are callable and not private
                # Exclude authentication methods to prevent infinite recursion
                if (
                    asyncio.iscoroutinefunction(attr)
                    and callable(attr)
                    and not name.startswith("_")
                    and "authenticate" not in name  # Exclude all authenticate methods
                ):
                    # Create a wrapped version that preserves the method binding
                    async def retry_wrapper(*args, **kwargs):
                        return await self._sparkfly_client._call_with_retry(
                            attr, *args, **kwargs
                        )

                    # Preserve the method name for better debugging and IDE discovery
                    retry_wrapper.__name__ = attr.__name__
                    retry_wrapper.__qualname__ = attr.__qualname__

                    return retry_wrapper

                return attr

        return RetryWrappedApi(self._api_client, self)

    async def __aenter__(self):
        """Async context manager entry."""
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        """Async context manager exit."""
        await self.close()

    async def close(self):
        """Close the underlying API client."""
        await self._api_client.close()

    @property
    def token(self) -> Optional[str]:
        """Get the current authentication token."""
        return self._token

    @property
    def token_expires_at(self) -> Optional[float]:
        """Get the token expiration timestamp."""
        return self._token_expires_at

    def is_token_valid(self) -> bool:
        """Check if the current token is still valid."""
        if not self._token or not self._token_expires_at:
            return False
        return time.time() < self._token_expires_at

    async def authenticate(self) -> str:
        """
        Authenticate with the Sparkfly API and get a token.

        Returns:
            The authentication token

        Raises:
            Exception: If authentication fails
        """
        try:
            # Determine the host index based on environment
            host_index = 1 if self.environment.lower() == "production" else 0

            # Request authentication token
            response = await self.auth.authenticate_with_http_info(
                _host_index=host_index
            )

            # Extract the token from the response headers
            if (
                hasattr(response, "headers")
                and response.headers
                and "X-Auth-Token" in response.headers
            ):
                self._token = response.headers["X-Auth-Token"]
            else:
                # Fallback: check if the token is in the API client configuration
                self._token = self._config.api_key.get("XAuthToken")

            if not self._token:
                raise Exception("No authentication token received from the API")

            # Set token expiration to 24 hours from now
            self._token_expires_at = time.time() + (24 * 60 * 60)  # 24 hours

            # Update the API client configuration with the token
            self._config.api_key["XAuthToken"] = self._token

            return self._token

        except Exception as e:
            raise Exception(f"Authentication failed: {e}")

    async def ensure_authenticated(self) -> str:
        """
        Ensure we have a valid authentication token.

        Returns:
            The authentication token
        """
        if not self.is_token_valid():
            return await self.authenticate()
        if not self._token:
            raise Exception("Authentication token is missing")
        return self._token

    async def _call_with_retry(self, api_method, *args, **kwargs):
        """
        Call an API method with automatic authentication and retry logic.

        Args:
            api_method: The async API method to call
            *args: Arguments to pass to the method
            **kwargs: Keyword arguments to pass to the method

        Returns:
            The API response
        """
        # Ensure we're authenticated
        await self.ensure_authenticated()

        # Apply retry logic
        last_exception = None
        delay = self.retry_config.base_delay

        for attempt in range(self.retry_config.max_retries + 1):
            try:
                return await api_method(*args, **kwargs)
            except self.retry_config.retry_on_exceptions as e:
                last_exception = e

                # Check if we should retry based on custom condition
                if (
                    self.retry_config.retry_condition
                    and not self.retry_config.retry_condition(e)
                ):
                    raise e

                # If this was the last attempt, raise the exception
                if attempt == self.retry_config.max_retries:
                    break

                # Wait before retrying
                await asyncio.sleep(delay)
                delay = min(
                    delay * self.retry_config.backoff_factor,
                    self.retry_config.max_delay,
                )

        # If we get here, all retries failed
        raise (
            last_exception
            if last_exception
            else Exception("API call failed after retries")
        )
